//	OWNER-DRAW LISTBOX/COMBOBOX CLASS LIBRARY
// COPYRIGHT 1995 JAMES J. HUANG
// ALL RIGHTS RESERVED
//
// WARRANTY: THIS CODE IS PROVIDED AS-IS. THE AUTHOR AND DISTRIBUTORS ARE
//           NOT LIABLE TO ANY DAMAGES THAT MAY INCUR OUT OF USING THIS 
//           CLASS LIBRARY.
//

#ifndef __HSTRING_H
#define __HSTRING_H

// FILE:     HSTRING.H
//
//	CONTENTS: class HString
//
// HString: almost the same as CString, except --
// 1) Cleared of MFC/OLE stuff (no AFX...);
// 2) Only large mode, no 'const char*' but 'LPCSTR'; no 'char*' but 'LPSTR';
// 3) LoadString() also takes hInst in HString;
// 4) No DBCS support.
//
// This class is used for non-MFC applications.
// If you are using MFC, may use #define HString CString in place of 
// #include "hstring.h", and get rid of "hstring.cpp" in your project.
//

#include <windows.h>
#include <string.h>

class HString
{
public:

// Constructors
	HString();
	HString(const HString& stringSrc);
	HString(char ch, int nRepeat = 1);
	HString(LPCSTR lpsz);
	HString(LPCSTR lpch, int nLength);
	~HString();

// Attributes & Operations

	// as an array of characters
	int GetLength() const;
	BOOL IsEmpty() const;
	void Empty();                       // free up the data

	char GetAt(int nIndex) const;       // 0 based
	char operator[](int nIndex) const;  // same as GetAt
	void SetAt(int nIndex, char ch);
	operator LPCSTR() const;       // as a C string

	// overloaded assignment
	const HString& operator=(const HString& stringSrc);
	const HString& operator=(char ch);
	const HString& operator=(LPCSTR psz);

	// string concatenation
	const HString& operator+=(const HString& s);
	const HString& operator+=(char ch);
	const HString& operator+=(LPCSTR psz);

	friend HString operator+(const HString& s1, const HString& s2);
	friend HString operator+(const HString& s, char ch);
	friend HString operator+(char ch, const HString& s);
	friend HString operator+(const HString& s, LPCSTR psz);
	friend HString operator+(LPCSTR psz, const HString& s);

	// string comparison
	int Compare(LPCSTR psz) const;         // straight character
	int CompareNoCase(LPCSTR psz) const;   // ignore case
	int Collate(LPCSTR psz) const;         // NLS aware

	// simple sub-string extraction
	HString Mid(int nFirst, int nCount) const;
	HString Mid(int nFirst) const;
	HString Left(int nCount) const;
	HString Right(int nCount) const;

	HString SpanIncluding(LPCSTR pszCharSet) const;
	HString SpanExcluding(LPCSTR pszCharSet) const;

	// upper/lower/reverse conversion
	void MakeUpper();
	void MakeLower();
	void MakeReverse();

	// searching (return starting index, or -1 if not found)
	// look for a single character match
	int Find(char ch) const;				// like "C" strchr
	int ReverseFind(char ch) const;
	int FindOneOf(LPCSTR pszCharSet) const;

	// look for a specific sub-string
	int Find(LPCSTR pszSub) const;			// like "C" strstr

	// Windows support
#ifdef _WINDOWS
	BOOL LoadString(HINSTANCE, UINT nID);	// 255 chars max
	void AnsiToOem();	// ANSI->OEM support (convert string in place)
	void OemToAnsi();   // ANSI<-OEM support (convert string in place)
#endif //_WINDOWS

	// Access to string implementation buffer as "C" character array
	LPSTR GetBuffer(int nMinBufLength);
	void ReleaseBuffer(int nNewLength = -1);
	LPSTR GetBufferSetLength(int nNewLength);

// Implementation
public:
	int GetAllocLength() const;
protected:
	// lengths/sizes in characters
	//  (note: an extra character is always allocated)
	LPSTR m_pchData;            // actual string (zero terminated)
	int m_nDataLength;          // does not include terminating 0
	int m_nAllocLength;         // does not include terminating 0

	// implementation helpers
	void Init();
	void AllocCopy(HString& dest, int nCopyLen, int nCopyIndex, int nExtraLen) const;
	void AllocBuffer(int nLen);
	void AssignCopy(int nSrcLen, LPCSTR pszSrcData);
	void ConcatCopy(int nSrc1Len, LPCSTR pszSrc1Data, int nSrc2Len, LPCSTR pszSrc2Data);
	void ConcatInPlace(int nSrcLen, LPCSTR pszSrcData);
	static void SafeDelete(LPSTR pch);
	static int SafeStrlen(LPCSTR psz);
};


// inline functions --

inline HString::HString() { Init(); }
inline HString::HString(const HString& stringSrc)
	{ stringSrc.AllocCopy(*this, stringSrc.m_nDataLength, 0, 0); }
inline HString::~HString() { SafeDelete(m_pchData); }
inline const HString& HString::operator =(const HString& stringSrc)
	{ AssignCopy(stringSrc.m_nDataLength, stringSrc.m_pchData); return *this; }
inline const HString& HString::operator =(LPCSTR psz)
	{ AssignCopy(SafeStrlen(psz), psz); return *this; }
inline const HString& HString::operator +=(LPCSTR psz)
	{ ConcatInPlace(SafeStrlen(psz), psz); return *this; }
inline const HString& HString::operator +=(char ch)
	{ ConcatInPlace(1, &ch); return *this; }
inline const HString& HString::operator +=(const HString& string)
	{ ConcatInPlace(string.m_nDataLength, string.m_pchData); return *this; }
inline int  HString::GetLength() const { return m_nDataLength; }
inline int  HString::GetAllocLength() const { return m_nAllocLength; }
inline BOOL HString::IsEmpty() const { return m_nDataLength == 0; }
inline HString::operator LPCSTR() const { return (LPCSTR)m_pchData; }
inline int  HString::SafeStrlen(LPCSTR psz) { return (psz == NULL) ? NULL : strlen(psz); }
inline int  HString::Compare(LPCSTR psz) const { return psz ? strcmp(m_pchData, psz) : 1; }
inline int  HString::CompareNoCase(LPCSTR psz) const { return psz ? stricmp(m_pchData, psz) : 1; }
inline int  HString::Collate(LPCSTR psz) const { return strcoll(m_pchData, psz); }
inline void HString::MakeUpper() { strupr(m_pchData); }
inline void HString::MakeLower() { strlwr(m_pchData); }
inline void HString::MakeReverse() { strrev(m_pchData); }
inline char HString::GetAt(int nIndex) const { return (*this)[nIndex]; }
inline char HString::operator[](int nIndex) const
	{
/*		ASSERT(nIndex >= 0);
		ASSERT(nIndex < m_nDataLength);
*/		return m_pchData[nIndex];
	}
inline void HString::SetAt(int nIndex, char ch)
	{
/*		ASSERT(nIndex >= 0);
		ASSERT(nIndex < m_nDataLength);
		ASSERT(ch != 0);
*/		m_pchData[nIndex] = ch;
	}

#ifdef _WINDOWS
inline void HString::AnsiToOem() { ::AnsiToOem(m_pchData, m_pchData); }
inline void HString::OemToAnsi() { ::OemToAnsi(m_pchData, m_pchData); }
#endif

inline BOOL operator==(const HString& s1, const HString& s2) { return s1.Compare(s2) == 0; }
inline BOOL operator==(const HString& s1, LPCSTR s2        ) { return s1.Compare(s2) == 0; }
inline BOOL operator==(LPCSTR s1,         const HString& s2) { return s2.Compare(s1) == 0; }
inline BOOL operator!=(const HString& s1, const HString& s2) { return s1.Compare(s2) != 0; }
inline BOOL operator!=(const HString& s1, LPCSTR s2        ) { return s1.Compare(s2) != 0; }
inline BOOL operator!=(LPCSTR s1,         const HString& s2) { return s2.Compare(s1) != 0; }
inline BOOL operator< (const HString& s1, const HString& s2) { return s1.Compare(s2) <  0; }
inline BOOL operator< (const HString& s1, LPCSTR s2        ) { return s1.Compare(s2) <  0; }
inline BOOL operator< (LPCSTR s1,         const HString& s2) { return s2.Compare(s1) >  0; }
inline BOOL operator> (const HString& s1, const HString& s2) { return s1.Compare(s2) >  0; }
inline BOOL operator> (const HString& s1, LPCSTR s2        ) { return s1.Compare(s2) >  0; }
inline BOOL operator> (LPCSTR s1,         const HString& s2) { return s2.Compare(s1) <  0; }
inline BOOL operator<=(const HString& s1, const HString& s2) { return s1.Compare(s2) <= 0; }
inline BOOL operator<=(const HString& s1, LPCSTR s2        ) { return s1.Compare(s2) <= 0; }
inline BOOL operator<=(LPCSTR s1,         const HString& s2) { return s2.Compare(s1) >= 0; }
inline BOOL operator>=(const HString& s1, const HString& s2) { return s1.Compare(s2) >= 0; }
inline BOOL operator>=(const HString& s1, LPCSTR s2        ) { return s1.Compare(s2) >= 0; }
inline BOOL operator>=(LPCSTR s1,         const HString& s2) { return s2.Compare(s1) <= 0; }

#endif
